<?php
/**
 * Zen
 *
 * @author    Zen
 * @copyright Copyright (c) 2021 Zen
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License ("AFL") v. 3.0
 *
 * https://www.zen.com/
 */

use Zen\Payment\CartData;
use Zen\Payment\Paywall;
use Zen\Payment\Util;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class ZencomPaymentModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_column_right
 */
class ZencomPaymentModuleFrontController extends ModuleFrontController
{

    /**
     * Initialize controller
     *
     * @see FrontController::init()
     */
    public function init()
    {
        $this->display_column_left = false;
        $this->display_column_right = false;
        parent::init();
    }

    /**
     * @throws NotSupportedHashMethod
     * @throws OrderIsNotAnArray
     */
    public function initContent()
    {
        parent::initContent();

        $cart = $this->context->cart;

        if (!$cart->date_upd) {
            Tools::redirect('/');

            return;
        }

        $zen = new Zencom();

        $cartId = $cart->id;

        $customer = new Customer((int)$cart->id_customer);

        $zen->validateOrder(
            (int)$cartId,
            Configuration::get('PAYMENT_ZEN_NEW_STATUS'),
            $cart->getOrderTotal(),
            $zen->displayName,
            null,
            [],
            null,
            false,
            $customer->secure_key
        );

        $orderId = Order::getOrderByCartId((int)$cartId);
        $order = new Order($orderId);

        $currencyInfo = Currency::getCurrency($order->id_currency);

        $paywall = new Paywall(
            Util::ENVIRONMENT_PRODUCTION,
            Configuration::get('ZEN_PAYWALL_SECRET'),
            Configuration::get('ZEN_TERMINAL_ID')
        );

        $total = Util::convertAmountToFractional($order->total_paid);

        $successUrl = $this->context->link->getPageLink('order-confirmation', true, null, [
            'id_cart' => $cart->id,
            'key' => $customer->secure_key,
            'id_module' => $zen->id,
            'id_order' => $orderId,
        ]);

        Logger::addLog($successUrl);

        $this->context->smarty->assign([
            'data' => $paywall->buildOrderForm(
                Paywall::prepareData(
                    $total,
                    $currencyInfo['iso_code'],
                    $orderId,
                    $customer->firstname,
                    $customer->lastname,
                    $customer->email,
                    $successUrl,
                    $this->context->link->getModuleLink('zencom', 'failure'),
                    $successUrl,
                    $this->context->link->getModuleLink('zencom', 'notification'),
                    $this->getCart($cart, $total)
                )
            ),
            'zen_msg_redirect_hint' => $this->module->l(
                'Please wait, in the next step you will choose a payment method',
                'payment'
            ),
        ]);

        $this->setTemplate('pay.tpl');
    }

    /**
     * @param Cart $cart
     * @param int $total
     *
     * @return array
     * @throws Exception
     */
    public function getCart($cart, $total)
    {

        $cartData = new CartData();
        $cartData->setAmount($total);

        foreach ($cart->getProducts() as $product) {
            $cartData->addItem(
                $product['name'],
                Util::convertAmountToFractional($product['price_wt']),
                $product['quantity'],
                Util::convertAmountToFractional($product['price_wt']) * $product['quantity']
            );
        }

        $carrier = new Carrier($cart->id_carrier);

        $shippingCost = Util::convertAmountToFractional($cart->getTotalShippingCost());

        $cartData->addItem(
            isset($carrier->name) ? $carrier->name : $this->module->l('Delivery cost'),
            $shippingCost,
            1,
            $shippingCost
        );

        $discount = Util::convertAmountToFractional($cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS));

        if ($discount > 0) {
            $cartData->addItem(
                $this->module->l('Discount'),
                -$discount,
                1,
                -$discount
            );
        }

        return $cartData->getCartData($this->module->l('Round'));
    }
}
