<?php

namespace Zen\Payment;

/**
 * Class Paywall
 *
 * @package Zen\Payment
 */
class Paywall
{
    const ROUTE_PAY = '/api/payment';

    /**
     * @var array
     */
    public static $serviceUrls = [
        Util::ENVIRONMENT_PRODUCTION => 'https://secure.zen.com',
        Util::ENVIRONMENT_SANDBOX => 'https://secure.zen-test.com',
        Util::ENVIRONMENT_LOCAL => 'http://127.0.0.1'
    ];
    /**
     * @var string
     */
    private $paywallSecret;

    /**
     * @var string
     */
    private $terminalId;

    /**
     * @var string
     */
    private $environment;

    /**
     * Payment constructor.
     *
     * @param string $environment
     * @param string $paywallSecret
     * @param string $terminalId
     */
    public function __construct($environment, $paywallSecret, $terminalId)
    {
        $this->environment = $environment;
        $this->paywallSecret = trim($paywallSecret);
        $this->terminalId = trim($terminalId);
    }

    /**
     * @param int $amount
     * @param string $currency
     * @param string $orderId
     * @param string $customerFirstName
     * @param string $customerLastName
     * @param string $customerEmail
     * @param string $urlSuccess
     * @param string $urlFailure
     * @param string $urlReturn
     * @param string $urlIpn
     * @param array $items
     *
     * @return array
     */
    public static function prepareData(
        $amount, $currency, $orderId, $customerFirstName, $customerLastName,
        $customerEmail, $urlSuccess, $urlFailure, $urlReturn, $urlIpn, $items
    )
    {
        $data = [];
        $data['amount'] = $amount;
        $data['currency'] = $currency;
        $data['merchantTransactionId'] = Util::generateMerchantTransactionId($orderId);
        $data['customer[firstName]'] = $customerFirstName;
        $data['customer[lastName]'] = $customerLastName;
        $data['customer[email]'] = $customerEmail;
        $data['urlSuccess'] = $urlSuccess;
        $data['urlFailure'] = $urlFailure;
        $data['urlReturn'] = $urlReturn;
        $data['customIpnUrl'] = $urlIpn;

        foreach ($items as $key => $value) {

            if (isset($value['name']) && $value['name']) {
                $data["items[$key][name]"] = $value['name'];
            }
            $data["items[$key][price]"] = $value['price'];
            $data["items[$key][quantity]"] = $value['quantity'];
            $data["items[$key][lineAmountTotal]"] = $value['lineAmountTotal'];
        }

        return $data;
    }

    /**
     * Creates full object with order data.
     *
     * @param array $orderData
     *
     * @return string
     */
    public function buildOrderForm($orderData)
    {
        return json_encode([
            'formObject' => $this->prepareOrderData($orderData),
            'urlObject' => $this->getTransactionCreateUrl($this->environment),
        ]);
    }

    /**
     * Simple function that adds signature
     *
     * @param array $order
     * @param string $hashMethod
     *
     * @return array
     */
    public function prepareOrderData($order, $hashMethod = 'sha256')
    {
        $order['terminalId'] = $this->terminalId;
        $order['signature'] = $this->createSignature($order, $this->paywallSecret, $hashMethod);

        return $order;
    }

    /**
     * @param array $orderData
     * @param string $serviceKey
     * @param string $hashMethod
     *
     * @return string|bool
     */
    private function createSignature($orderData, $serviceKey, $hashMethod = 'sha256')
    {

        $isHashMethodSupported = Util::getHashMethod($hashMethod);

        if (!$isHashMethodSupported
            || !is_array($orderData)) {
            return false;
        }

        return Util::hashSignature($hashMethod, self::prepareHashData($orderData), $serviceKey) . ';' . $hashMethod;
    }

    /**
     * @param array $data
     * @param string $prefix
     *
     * @return string
     */
    public static function prepareHashData($data, $prefix = '')
    {

        ksort($data);

        $hashData = [];
        foreach ($data as $key => $value) {

            if ($prefix) {
                $key = $prefix . '[' . $key . ']';
            }
            if (is_array($value)) {
                $hashData[] = self::prepareHashData($value, $key);
            } else {
                $hashData[] = $key . '=' . $value;
            }
        }

        return implode('&', $hashData);
    }

    /**
     * @param string $environment
     *
     * @return string
     */
    public function getTransactionCreateUrl($environment)
    {

        $baseUrl = self::getServiceUrl($environment);

        if ($baseUrl) {
            return $baseUrl . self::ROUTE_PAY;
        }

        return '';
    }

    /**
     * @param string $environment
     *
     * @return string
     */
    public static function getServiceUrl($environment)
    {

        if (isset(self::$serviceUrls[$environment])) {
            return self::$serviceUrls[$environment];
        }

        return '';
    }
}
